# -*- coding: utf-8 -*-
"""
This module contains metadata about the project.
"""
from __future__ import unicode_literals

title = 'swid_generator'
version = '0.2.0'
description = 'Application which generates SWID-Tags from Linux installed packages, ' \
              'using tools like DPGK or RPM.'
authors = 'Christian Fässler, Danilo Bargen, Jonas Furrer'
license = 'MIT'
